@@ -1,5 +1,5 @@
 ## R routines for gam fitting with calculation of derivatives w.r.t. sp.s
-## (c) Simon Wood 2004-2009
+## (c) Simon Wood 2004-2013
 
 ## These routines are for type 3 gam fitting. The basic idea is that a P-IRLS
 ## is run to convergence, and only then is a scheme for evaluating the 
@@ -39,7 +39,7 @@ gam.reparam <- function(rS,lsp,deriv)
                   d.tol = as.double(d.tol),
                   r.tol = as.double(r.tol),
                   fixed_penalty = as.integer(fixed.penalty))
-  S <- matrix(oo$S,q,q)
+  S <- matrix(oo$S,q,q)  
   S <- (S+t(S))*.5
   p <- abs(diag(S))^.5            ## by Choleski, p can not be zero if S +ve def
   p[p==0] <- 1                    ## but it's possible to make a mistake!!
@@ -81,32 +81,40 @@ gam.fit3 <- function (x, y, sp, Eb,UrS=list(),
             mustart = NULL, offset = rep(0, nobs),U1=diag(ncol(x)), Mp=-1, family = gaussian(), 
             control = gam.control(), intercept = TRUE,deriv=2,
             gamma=1,scale=1,printWarn=TRUE,scoreType="REML",null.coef=rep(0,ncol(x)),
-            pearson.extra=0,dev.extra=0,n.true=-1,...) 
- 
-## Version with new reparameterization and truncation strategy. 
-## ISSUES: 
-## 1. More efficient use of re-parameterization strategy could be employed, 
-##    as repara is O(nq^2). Could pass in a repara object, containing sp's
-##    at last repara, X, rS, E, Eb in transformed form + T. If object is NULL
-##    that would signal need to create it. It would only be re-created if the log sps
-##    have moved far enough that some component is more than 3? from sps at last re-para.
-##    repara object would be returned at end of routine. 
-##    --- note that this would require det|S| calculation to be performed separately 
-##        in cases where re-parameterization is not done. 
-##    --- not clear if this is really worth the effort: point is to avoid forming X%*%T
-##        every time routine is called, but this is only saving one O(np^2) operation 
-##        from several to many.
-##    --- also this idea is difficult to make work with optimizers other than mine.
-## This version is designed to allow iterative weights to be negative. This means that 
-## it deals with weights, rather than sqrt weights.
-## deriv, sp, S, rS, H added to arg list. 
-## need to modify family before call.
-## pearson.extra is an extra component to add to the pearson statistic in the P-REML/P-ML 
-## case, only.
-## dev.extra is an extra component to add to the deviance in the REML and ML cases only.
-## Similarly, n.true is to be used in place of the length(y) in ML/REML calculations,
-## and the scale.est only.
-{   if (family$link==family$canonical) fisher <- TRUE else fisher=FALSE 
+            pearson.extra=0,dev.extra=0,n.true=-1,...) {
+## Inputs:
+## * x model matrix
+## * y response
+## * sp log smoothing parameters
+## * Eb square root of nicely balanced total penalty matrix used for rank detection
+## * UrS list of penalty square roots in range space of overall penalty. UrS[[i]]%*%t(UrS[[i]]) 
+##   is penalty. See 'estimate.gam' for more.
+## * weights prior weights (reciprocal variance scale)
+## * start initial values for parameters
+## * etastart initial values for eta
+## * mustart initial values for mu... only one of last## * control - control list.
+## * intercept - indicates whether model has one.
+## * deriv - order 0,1 or 2 derivatives are to be returned (lower is cheaper!)
+## * gamma - multiplier for effective degrees of freedom in GCV/UBRE.
+## * scale parameter. Negative signals to estimate.
+## * printWarn print or supress?
+## * scoreType - type of smoothness selection to use.
+## * null.coef - coefficients for a null model, in order to be able to check for immediate 
+##   divergence.
+## * pearson.extra is an extra component to add to the pearson statistic in the P-REML/P-ML 
+##   case, only.
+## * dev.extra is an extra component to add to the deviance in the REML and ML cases only.
+## * n.true is to be used in place of the length(y) in ML/REML calculations,
+##   and the scale.est only.
+## 
+## Version with new reparameterization and truncation strategy. Allows iterative weights 
+## to be negative. Basically the workhorse routine for Wood (2011) JRSSB.
+## A much modified version of glm.fit. Purpose is to estimate regression coefficients 
+## and compute a smoothness selection score along with its derivatives.
+##
+    if (control$trace) { t0 <- proc.time();tc <- 0} 
+
+    if (family$link==family$canonical) fisher <- TRUE else fisher=FALSE 
     ## ... if canonical Newton = Fisher, but Fisher cheaper!
     if (scale>0) scale.known <- TRUE else scale.known <- FALSE
     if (!scale.known&&scoreType%in%c("REML","ML")) { ## the final element of sp is actually log(scale)
@@ -152,10 +160,13 @@ gam.fit3 <- function (x, y, sp, Eb,UrS=list(),
       T[1:ncol(rp$Qs),1:ncol(rp$Qs)] <- rp$Qs
       T <- U1%*%T ## new params b'=T'b old params
     
-      null.coef <- t(T)%*%null.coef
-
-      x <- x%*%T   ## model matrix
+      null.coef <- t(T)%*%null.coef  
+     
+      ## form x%*%T in parallel 
+      x <- .Call(C_mgcv_pmmult2,x,T,0,0,control$nthreads)
+      ## x <- x%*%T   ## model matrix 0(nq^2)
    
+
       rS <- list()
       for (i in 1:length(UrS)) {
         rS[[i]] <- rbind(rp$rS[[i]],matrix(0,Mp,ncol(rp$rS[[i]])))
@@ -194,7 +205,7 @@ gam.fit3 <- function (x, y, sp, Eb,UrS=list(),
         valideta <- function(eta) TRUE
     validmu <- family$validmu
     if (is.null(validmu)) 
-        validmu <- function(mu) TRUE
+       validmu <- function(mu) TRUE
     if (is.null(mustart)) {
         eval(family$initialize)
     }
@@ -302,19 +313,22 @@ gam.fit3 <- function (x, y, sp, Eb,UrS=list(),
             ## Here a Fortran call has been replaced by pls_fit1 call
            
             if (sum(good)<ncol(x)) stop("Not enough informative observations.")
-           
+            if (control$trace) t1 <- proc.time()
             oo <- .C(C_pls_fit1,y=as.double(z),X=as.double(x[good,]),w=as.double(w),
                      E=as.double(Sr),Es=as.double(Eb),n=as.integer(sum(good)),
                      q=as.integer(ncol(x)),rE=as.integer(rows.E),eta=as.double(z),
-                     penalty=as.double(1),rank.tol=as.double(rank.tol))
+                     penalty=as.double(1),rank.tol=as.double(rank.tol),nt=as.integer(control$nthreads))
+            if (control$trace) tc <- tc + sum((proc.time()-t1)[c(1,4)])
 
             if (!fisher&&oo$n<0) { ## likelihood indefinite - switch to Fisher for this step
               z <- (eta - offset)[good] + (yg - mug)/mevg
               w <- (weg * mevg^2)/var.mug
+              if (control$trace) t1 <- proc.time()
               oo <- .C(C_pls_fit1,y=as.double(z),X=as.double(x[good,]),w=as.double(w),
                        E=as.double(Sr),Es=as.double(Eb),n=as.integer(sum(good)),
                        q=as.integer(ncol(x)),rE=as.integer(rows.E),eta=as.double(z),
-                       penalty=as.double(1),rank.tol=as.double(rank.tol))
+                       penalty=as.double(1),rank.tol=as.double(rank.tol),nt=as.integer(control$nthreads))
+              if (control$trace) tc <- tc + sum((proc.time()-t1)[c(1,4)])
             }
 
             start <- oo$y[1:ncol(x)];
@@ -395,7 +409,7 @@ gam.fit3 <- function (x, y, sp, Eb,UrS=list(),
              }
              while (pdev -old.pdev > div.thresh)  
              { ## step halve until pdev <= old.pdev
-                if (ii > 200) 
+                if (ii > 100) 
                    stop("inner loop 3; can't correct step size")
                 ii <- ii + 1
                 start <- (start + coefold)/2 
@@ -517,7 +531,7 @@ gam.fit3 <- function (x, y, sp, Eb,UrS=list(),
        if (scoreType%in%c("ML","P-ML")) {REML <- -1;remlInd <- 0} 
 
        if (REML==0) rSncol <- unlist(lapply(rS,ncol)) else rSncol <- unlist(lapply(UrS,ncol))
-
+       if (control$trace) t1 <- proc.time()
        oo <- .C(C_gdi1,X=as.double(x[good,]),E=as.double(Sr),Eb = as.double(Eb), 
                 rS = as.double(unlist(rS)),U1=as.double(U1),sp=as.double(exp(sp)),
                 z=as.double(z),w=as.double(w),wf=as.double(wf),alpha=as.double(alpha),
@@ -532,9 +546,11 @@ gam.fit3 <- function (x, y, sp, Eb,UrS=list(),
                 p=as.integer(ncol(x)),M=as.integer(nSp),Mp=as.integer(Mp),Enrow = as.integer(rows.E),
                 rSncol=as.integer(rSncol),deriv=as.integer(deriv.sp),
                 REML = as.integer(REML),fisher=as.integer(fisher),
-                fixed.penalty = as.integer(rp$fixed.penalty))      
-       
-         if (control$trace) cat("done!\n")
+                fixed.penalty = as.integer(rp$fixed.penalty),nthreads=as.integer(control$nthreads))      
+         if (control$trace) { 
+           tg <- sum((proc.time()-t1)[c(1,4)])
+           cat("done!\n")
+         }
  
          rV <- matrix(oo$rV,ncol(x),ncol(x)) ## rV%*%t(rV)*scale gives covariance matrix 
          
@@ -691,7 +707,11 @@ gam.fit3 <- function (x, y, sp, Eb,UrS=list(),
     nulldf <- n.ok - as.integer(intercept)
    
     aic.model <- aic(y, n, mu, weights, dev) # note: incomplete 2*edf needs to be added
-
+    if (control$trace) {
+      t1 <- proc.time()
+      at <- sum((t1-t0)[c(1,4)])
+      cat("Proportion time in C: ",(tc+tg)/at," ls:",tc/at," gdi:",tg/at,"\n")
+    } 
    
     list(coefficients = coef, residuals = residuals, fitted.values = mu, 
          family = family, linear.predictors = eta, deviance = dev, 
@@ -708,8 +728,10 @@ gam.fit3.post.proc <- function(X,object) {
 ## get edf array and covariance matrices after a gam fit. 
 ## X is original model matrix
   Vb <- object$rV%*%t(object$rV)*object$scale ## Bayesian cov.
-  PKt <- object$rV%*%t(object$K)
-  F <- PKt%*%(sqrt(object$weights)*X)
+  # PKt <- object$rV%*%t(object$K)
+  PKt <- .Call(C_mgcv_pmmult2,object$rV,object$K,0,1,object$control$nthreads)
+  # F <- PKt%*%(sqrt(object$weights)*X)
+  F <- .Call(C_mgcv_pmmult2,PKt,sqrt(object$weights)*X,0,0,object$control$nthreads)
   edf <- diag(F) ## effective degrees of freedom
   edf1 <- 2*edf - rowSums(t(F)*F) ## alternative
   ## edf <- rowSums(PKt*t(sqrt(object$weights)*X))
@@ -719,8 +741,10 @@ gam.fit3.post.proc <- function(X,object) {
   ## get QR factor R of WX - more efficient to do this
   ## in gdi_1 really, but that means making QR of augmented 
   ## a two stage thing, so not clear cut...
-  qrx <- qr(sqrt(object$weights)*X,LAPACK=TRUE)
-  R <- qr.R(qrx);R[,qrx$pivot] <- R
+  #qrx <- qr(sqrt(object$weights)*X,LAPACK=TRUE,tol=0)
+  #R <- qr.R(qrx);R[,qrx$pivot] <- R
+  qrx <- pqr(sqrt(object$weights)*X,object$control$nthreads)
+  R <- pqr.R(qrx);R[,qrx$pivot] <- R
   ##bias = as.numeric(object$coefficients - F%*%object$coefficients)
   list(Vb=Vb,Ve=Ve,edf=edf,edf1=edf1,hat=hat,F=F,R=R)
 }
@@ -1232,7 +1256,7 @@ newton <- function(lsp,X,y,Eb,UrS,L,lsp0,offset,U1,Mp,family,weights,
   else if (i==200) ct <- "iteration limit reached" 
   else ct <- "full convergence"
   list(score=score,lsp=lsp,lsp.full=L%*%lsp+lsp0,grad=grad,hess=hess,iter=i,conv =ct,score.hist = score.hist[!is.na(score.hist)],object=b)
-}
+} ## newton
 
 bfgs0 <- function(lsp,X,y,Eb,UrS,L,lsp0,offset,U1,Mp,family,weights,
                    control,gamma,scale,conv.tol=1e-6,maxNstep=5,maxSstep=2,
@@ -2039,6 +2063,19 @@ fix.family.ls<-function(fam)
   stop("family not recognised")
 }
 
+fix.family <- function(fam) {
+## allows families to be patched...
+   if (fam$family[1]=="gaussian") { ## sensible starting values given link...
+     fam$initialize <- expression({
+     n <- rep.int(1, nobs)
+     if (family$link == "inverse") mustart <- y + (y==0)*sd(y)*.01 else
+     if (family$link == "log") mustart <- pmax(y,.01*sd(y)) else
+     mustart <- y
+     })
+  }
+  fam
+}
+
 
 negbin <- function (theta = stop("'theta' must be specified"), link = "log") { 
 ## modified from Venables and Ripley's MASS library to work with gam.fit3,
@@ -2117,7 +2154,7 @@ negbin <- function (theta = stop("'theta' must be specified"), link = "log") {
         linkinv = stats$linkinv, variance = variance,dvar=dvar,d2var=d2var,d3var=d3var, dev.resids = dev.resids,
         aic = aic, mu.eta = stats$mu.eta, initialize = initialize,ls=ls,
         validmu = validmu, valideta = stats$valideta,getTheta = getTheta,qf=qf,rd=rd,canonical="log"), class = "family")
-}
+} ## negbin
 
 
 
@@ -2268,7 +2305,7 @@ Tweedie <- function(p=1,link=power(0)) {
        if (!is.null(stats$name))
           linktemp <- stats$name
         } else {
-            stop(gettextf("link \"%s\" not available for poisson family.",
+            stop(gettextf("link \"%s\" not available for Tweedie family.",
                 linktemp, collapse = ""),domain = NA)
         }
     }
