@@ -1,4 +1,5 @@
-/* Convenient C wrappers for calling LAPACK and LINPACK 
+/* Convenient C wrappers for calling LAPACK and LINPACK + other matrix routines using same 
+   packing format....
 
    See R-x/include/R_ext/Lapack.h...
 */
@@ -7,13 +8,17 @@
 #include <stdio.h>
 #include <math.h>
 #include <R.h>
+#include <Rinternals.h>
 #include <R_ext/Linpack.h> /* only needed for pivoted chol - see note in mgcv_chol */
 #include <R_ext/Lapack.h>
 #include <R_ext/BLAS.h>
+#include <Rconfig.h>
+#ifdef SUPPORT_OPENMP
+#include <omp.h>
+#endif
+
 /*#include <dmalloc.h>*/
 
-
-
 void dump_mat(double *M,int *r,int*c,const char *path) {
   /* dump r by c matrix M to path - intended for debugging use only */
   FILE *mf;
@@ -49,6 +54,7 @@ void read_mat(double *M,int *r,int*c,char *path) {
  } else {
    j=fread(r,sizeof(int),1,mf); j=fread(c,sizeof(int),1,mf);
    j=fread(M,sizeof(double),*r * *c,mf);
+   if (j!= *r * *c)  Rprintf("\nfile dim problem\n");
  }
  fclose(mf);
 }
@@ -136,9 +142,11 @@ void mgcv_mmult0(double *A,double *B,double *C,int *bt,int *ct,int *r,int *c,int
 } /* end mgcv_mmult0*/
 
 void mgcv_mmult(double *A,double *B,double *C,int *bt,int *ct,int *r,int *c,int *n) {
-  /* BLAS version A is c (result), B is a, C is b, bt is transa ct is transb 
+  /* Forms A = B C transposing B and C according to bt and ct.
+     A is r by c. Common dimension is n;
+     BLAS version A is c (result), B is a, C is b, bt is transa ct is transb 
      r is m, c is n, n is k.
-     Does nothing if r,c or n <= zero.
+     Does nothing if r,c or n <= zero. 
   */
   char transa='N',transb='N';
   int lda,ldb,ldc;
@@ -163,6 +171,280 @@ void mgcv_mmult(double *A,double *B,double *C,int *bt,int *ct,int *r,int *c,int
 		B, &lda,C, &ldb,&beta, A, &ldc);
 } /* end mgcv_mmult */
 
+SEXP mgcv_pmmult2(SEXP b, SEXP c,SEXP bt,SEXP ct, SEXP nthreads) {
+/* parallel matrix multiplication using .Call interface */
+  double *A,*B,*C;
+  int r,col,n,nt,Ct,Bt,m;
+  SEXP a; 
+ 
+  nt = asInteger(nthreads);
+  Bt = asInteger(bt);
+  Ct = asInteger(ct);
+ 
+  if (Bt) {
+    r = ncols(b);
+    n = nrows(b);
+  } else {
+    r = nrows(b);
+    n = ncols(b);
+  }
+ 
+  if (Ct) col = nrows(c);
+  else col = ncols(c);
+  /* Rprintf("nt = %d, Bt = %d, Ct = %d, r = %d, c = %d, n = %d\n",nt,Bt,Ct,r,col,n);*/
+  B = REAL(b);C=REAL(c);
+  a = PROTECT(allocMatrix(REALSXP,r,col));
+  A = REAL(a);
+  #ifdef SUPPORT_OPENMP
+  m = omp_get_num_procs(); /* detected number of processors */
+  if (nt > m || nt < 1) nt = m; /* no point in more threads than m */
+  #else
+  nt = 1; /* no openMP support - turn off threading */
+  #endif
+  mgcv_pmmult(A,B,C,&Bt,&Ct,&r,&col,&n,&nt);
+  UNPROTECT(1);
+  return(a);
+} /* mgcv_pmmult2 */
+
+
+int mgcv_piqr(double *x,int n, int p, double *beta, int *piv, int nt) {
+/* Do the work for parallel QR: 
+   Key is to inline the expensive step that is parallelized, noting
+   that housholder ops are not in BLAS, but are LAPACK auxilliary 
+   routines (so no loss in re-writing)
+*/
+
+  int i,k,r,nh,j,one=1,cpt,nth,cpf,ii;
+  double *c,*p0,*p1,*p2,xx,tau,*work,zz,*v,*z,*z1,br;
+  /* const char side = 'L';*/
+
+  #ifndef SUPPORT_OPENMP
+  nt = 1;
+  #endif
+
+  c =(double *)R_chk_calloc((size_t) p,sizeof(double)); 
+  work =(double *)R_chk_calloc((size_t) p*nt,sizeof(double));
+  k=0;tau=0.0;
+  /* find column norms O(np) */
+  for (p0=x,i=0;i<p;i++) {
+    piv[i] = i;
+    for (xx=0.0,p1=p0 + n;p0<p1;p0++) xx += *p0 * *p0;
+    c[i] = xx;
+    if (xx>tau) {tau=xx;k=i;}
+  }
+  r = -1;
+  nh = n; /* householder length */
+  
+  while (tau > 0) {
+    r++;
+    i=piv[r]; piv[r] = piv[k];piv[k] = i;
+    /* swap r with k O(n) */
+    xx = c[r];c[r] = c[k];c[k] = xx;
+    for (p0 = x + n * r, p1 = x + n * k,p2 = p0 + n;p0<p2;p0++,p1++) {
+          xx = *p0; *p0 = *p1; *p1 = xx;
+    }
+    /* now generate the householder reflector for column r O(n)*/
+    p0 = x + r * n + r; /* first element of column */
+    p1 = p0 + 1; /* remaining elements of column */
+    xx = *p0;
+    F77_CALL(dlarfg)(&nh,&xx,p1,&one,beta+r);
+    
+    /* next apply the rotation to the remaining columns of x O(np) */
+       
+    *p0 = 1.0;j=p-r-1;
+    
+    /* now distribute the j columns between nt threads */
+    if (j) {
+          cpt = j / nt; /* cols per thread */
+          if (cpt * nt < j) cpt++; 
+          nth = j/cpt; /* actual number of threads */
+          if (nth * cpt < j) nth++;
+          cpf = j - cpt * (nth-1); /* columns on final block */
+    } else nth=cpf=cpt=0;
+
+    br = beta[r];
+    #ifdef SUPPORT_OPENMP
+    #pragma omp parallel private(i,j,p1,v,z,z1,zz,ii) num_threads(nt)
+    #endif
+    { j = cpt;
+      if (j) {        
+        #ifdef SUPPORT_OPENMP
+        #pragma omp for
+        #endif
+        for (i=0;i<nth;i++) {
+	    if (i == nth-1) j = cpf;
+            p1 = p0 + n + n * cpt *i; 
+            z1=p1+nh;
+            for (ii =0;ii<j;ii++,p1+=n,z1+=n) {
+            for (zz=0.0,v=p0,z=p1;z<z1;z++,v++) zz += *z * *v * br;
+            for (z=p1,v=p0;z<z1;z++,v++) *z -= zz * *v;
+	   }
+	   /*F77_CALL(dlarfx)(&side, &nh, &j, p0, beta+r, p0 + n + n * cpt * i , &n , work + p * i);*/
+        }
+      } /* if (j) */
+    } /* end parallel */ 
+    nh--;
+    *p0 = xx;
+    /* update c, get new k... */
+    k = r + 1;
+    for (tau=0.0,p0+=n,i=r+1;i<p;i++,p0+=n) { 
+          c[i] -= *p0 * *p0;
+          if (c[i]>tau) { 
+            tau = c[i];
+            k=i;
+          }
+     }
+     if (r==n-1) tau = 0.0;
+    
+    } /* end while (tau > 0) */
+  
+  R_chk_free(c); R_chk_free(work);
+  return(r+1);
+  } /* mgcv_piqr */
+
+SEXP mgcv_Rpiqr(SEXP X, SEXP BETA,SEXP PIV,SEXP NT) {
+/* routine to QR decompose N by P matrix X with pivoting using routine
+   dlarg to generate housholder. This is direct 
+   implementation of 5.4.1 from Golub and van Loan (with correction 
+   of the pivot pivoting!). */
+  int n,p,nt,*piv,r,*rrp;
+  double *x,*beta;
+  SEXP rr;
+  nt = asInteger(NT);
+  n = nrows(X);
+  p = ncols(X);
+  x = REAL(X);beta = REAL(BETA);
+  piv = INTEGER(PIV);
+  r = mgcv_piqr(x,n,p,beta,piv,nt);
+  /* should return rank (r+1) */
+  rr = PROTECT(allocVector(INTSXP, 1));
+  rrp = INTEGER(rr);
+  *rrp = r;
+  UNPROTECT(1);
+  return(rr);
+
+} /* mgcv_piqr */
+
+void mgcv_pmmult(double *A,double *B,double *C,int *bt,int *ct,int *r,int *c,int *n,int *nt) {
+  /* 
+     Forms r by c product, A, of B and C, transposing each according to bt and ct.
+     n is the common dimension of the two matrices, which are stored in R 
+     default column order form.
+   
+     This version uses openMP parallelization. nt is number of threads to use. 
+     The strategy is rather simple, and this routine is really only useful when 
+     B and C have numbers of rows and columns somewhat higher than the number of 
+     threads. Assumes number of threads already set on entry and nt reset to 1
+     if no openMP support.
+
+     BLAS version A is c (result), B is a, C is b, bt is transa ct is transb 
+     r is m, c is n, n is k.
+     Does nothing if r,c or n <= zero. 
+  */
+  char transa='N',transb='N';
+  int lda,ldb,ldc,cpt,cpf,c1,i,nth;
+  double alpha=1.0,beta=0.0;
+  if (*r<=0||*c<=0||*n<=0) return;
+  if (B==C) { /* this is serial, unfortunately. note case must be caught as B can be re-ordered!  */
+    if (*bt&&(!*ct)&&(*r==*c)) { getXtX(A,B,n,r);return;} 
+    else if (*ct&&(!*bt)&&(*r==*c)) { getXXt(A,B,c,n);return;}
+  }
+  #ifndef SUPPORT_OPENMP
+  *nt = 1;
+  #endif
+  if (*nt == 1) {
+    mgcv_mmult(A,B,C,bt,ct,r,c,n); /* use single thread version */
+    return;
+  }
+  if (*bt) { /* so B is n by r */
+    transa = 'T';  
+    lda = *n;
+  } else  lda = *r; /* B is r by n */
+
+  if (*ct) { /* C is c by n */ 
+    transb = 'T';
+    ldb = *c;
+  } else ldb = *n; /* C is n by c */
+
+  ldc = *r;
+
+  if (*ct) { /* have to split on B, which involves re-ordering */
+    if (*bt) { /* can split on columns of n by r matrix B, but A then needs re-ordering */
+      cpt = *r / *nt; /* cols per thread */
+      if (cpt * *nt < *r) cpt++; 
+      nth = *r/cpt;
+      if (nth * cpt < *r) nth++;
+      cpf = *r - cpt * (nth-1); /* columns on final block */
+      #ifdef SUPPORT_OPENMP
+      #pragma omp parallel private(i,c1) num_threads(nth)
+      #endif
+      { /* open parallel section */
+        c1 = cpt;
+        #ifdef SUPPORT_OPENMP
+        #pragma omp for
+        #endif
+        for (i=0;i<nth;i++) {
+          if (i == nth-1) c1 = cpf;
+          /* note integer after each matrix is its leading dimension */
+          if (c1>0) F77_CALL(dgemm)(&transa,&transb,&c1,c,n, &alpha,
+		B + i * cpt * *n, n ,C, c,&beta, A + i * cpt * *c, &c1);
+        }
+      } /* parallel section ends */
+      /* now re-order the r by c matrix A, which currently contains the sequential
+         blocks corresponding to each cpt rows of A */
+      row_block_reorder(A,r,c,&cpt,bt); /* bt used here for 'reverse' as it contains a 1 */
+    } else { /* worst case - have to re-order r by n mat B and then reverse re-ordering of B and A at end */
+      cpt = *r / *nt; /* cols per thread */
+      if (cpt * *nt < *r) cpt++; 
+      nth = *r/cpt;
+      if (nth * cpt < *r) nth++;
+      cpf = *r - cpt * (nth-1); /* columns on final block */
+      /* re-order cpt-row blocks of B into sequential cpt by n matrices (in B) */ 
+      row_block_reorder(B,r,n,&cpt,bt); /* bt contains a zero - forward mode here */
+      #ifdef SUPPORT_OPENMP
+      #pragma omp parallel private(i,c1) num_threads(nth)
+      #endif
+      { /* open parallel section */
+        c1 = cpt;
+        #ifdef SUPPORT_OPENMP
+        #pragma omp for
+        #endif
+        for (i=0;i<nth;i++) {
+          if (i == nth-1) c1 = cpf;
+          if (c1>0) F77_CALL(dgemm)(&transa,&transb,&c1,c,n, &alpha,
+		B + i * cpt * *n, &c1,C,c,&beta, A + i * cpt * *c, &c1);
+        }
+      } /* parallel ends */
+      /* now reverse the re-ordering */
+      row_block_reorder(B,r,n,&cpt,ct);
+      row_block_reorder(A,r,c,&cpt,ct);
+    }
+  } else { /* can split on columns of n by c matrix C, which avoids re-ordering */
+    cpt = *c / *nt; /* cols per thread */
+    if (cpt * *nt < *c) cpt++;
+    nth = *c/cpt;
+    if (nth * cpt < *c) nth++;
+    cpf = *c - cpt * (nth-1); /* columns on final block */
+    #ifdef SUPPORT_OPENMP
+    #pragma omp parallel private(i,c1) num_threads(*nt)
+    #endif
+    { /* open parallel section */
+      c1 = cpt;
+      #ifdef SUPPORT_OPENMP
+      #pragma omp for
+      #endif
+      for (i=0;i< nth;i++) {
+        if (i == nth-1) c1 = cpf; /* how many columns in this block */
+        if (c1>0) F77_CALL(dgemm)(&transa,&transb,r,&c1,n, &alpha,
+		B, &lda,C + i * *n * cpt, &ldb,&beta, A + i * *r * cpt, &ldc);
+      }
+    } /* end parallel */
+  }
+
+} /* end mgcv_pmmult */
+
+
+
 void getXtX0(double *XtX,double *X,int *r,int *c)
 /* form X'X (nearly) as efficiently as possible - BLAS free*/
 { double *p0,*p1,*p2,*p3,*p4,x;
@@ -226,7 +508,7 @@ void getXtWX(double *XtWX, double *X,double *w,int *r,int *c,double *work)
 */ 
 { int i,j,one=1;
   char trans='T';
-  double *p,*p1,*p2,*pX0,xx,*w2,alpha=1.0,beta=0.0;
+  double *p,*p1,*p2,*pX0,xx=0.0,*w2,alpha=1.0,beta=0.0;
   pX0=X;
   w2 = XtWX; /* use first column as work space until end */
   for (i=0;i< *c;i++) { 
@@ -289,11 +571,11 @@ void mgcv_chol(double *a,int *pivot,int *n,int *rank)
 */
 { double *work,*p1,*p2,*p;
   int piv=1;
-  work=(double *)calloc((size_t) *n,sizeof(double));
+  work=(double *)R_chk_calloc((size_t) *n,sizeof(double));
   F77_CALL(dchdc)(a,n,n,work,pivot,&piv,rank);
   /* zero stuff below the leading diagonal */
   for (p2=a+ *n,p1=a+1;p2<a+ *n * *n;p1+= *n+1,p2+= *n) for (p=p1;p<p2;p++) *p=0.0;
-  free(work);
+  R_chk_free(work);
 }
 
 void mroot(double *A,int *rank,int *n)
@@ -311,11 +593,11 @@ void mroot(double *A,int *rank,int *n)
 */
 { int *pivot,erank,i,j;
   double *pi,*pj,*p0,*p1,*B;
-  pivot=(int *)calloc((size_t)*n,sizeof(int));
+  pivot=(int *)R_chk_calloc((size_t)*n,sizeof(int));
   mgcv_chol(A,pivot,n,&erank);
   if (*rank<=0) *rank = erank;
   /* unscramble the returned choleski factor */ 
-  B=calloc((size_t)(*n * *n),sizeof(double));
+  B=R_chk_calloc((size_t)(*n * *n),sizeof(double));
   /* copy A to B and zero A */
   for (p0=A,p1=B,i=0;i< *n;i++,p0+= *n,p1+= *n)
   for (pi=p0,pj=p1;pi<=p0+i;pi++,pj++) { *pj = *pi; *pi=0.0;}
@@ -325,7 +607,7 @@ void mroot(double *A,int *rank,int *n)
   /* remove trailing rows from choleski factor */
   for (pi=A,p0=A,i=0;i< *n;i++,p0+= *n)
   for (pj=p0;pj<p0+ *rank;pj++,pi++) *pi = *pj; 
-  free(pivot);free(B);
+  R_chk_free(pivot);R_chk_free(B);
 }
 
 
@@ -343,11 +625,11 @@ void mgcv_svd(double *x,double *u,double *d,int *r,int *c)
   		   &work1, &lwork, &info);
   lwork=(int)floor(work1);
   if (work1-lwork>0.5) lwork++;
-  work=(double *)calloc((size_t)lwork,sizeof(double));
+  work=(double *)R_chk_calloc((size_t)lwork,sizeof(double));
   /* actual call */
   F77_CALL(dgesvd)(&jobu,&jobvt, r, c, x, &lda, d, u, &ldu, vt,&ldvt,
   		   work, &lwork, &info);
-  free(work);
+  R_chk_free(work);
 }
 
 void mgcv_svd_full(double *x,double *vt,double *d,int *r,int *c)
@@ -376,11 +658,11 @@ matrix(um[[2]],q,q);er$v
   		   &work1, &lwork, &info);
   lwork=(int)floor(work1);
   if (work1-lwork>0.5) lwork++;
-  work=(double *)calloc((size_t)lwork,sizeof(double));
+  work=(double *)R_chk_calloc((size_t)lwork,sizeof(double));
   /* actual call */
   F77_CALL(dgesvd)(&jobu,&jobvt, r, c, x, &lda, d, u, &ldu, vt,&ldvt,
   		   work, &lwork, &info);
-  free(work);
+  R_chk_free(work);
 }
 
 
@@ -400,10 +682,10 @@ void mgcv_td_qy(double *S,double *tau,int *m,int *n, double *B,int *left,int *tr
   /* workspace query ... */
   F77_CALL(dormtr)(&side,&uplo,&trans,m,n,S,&nq,tau,B,m,&work1,&lwork,&info);
   lwork=(int)floor(work1);if (work1-lwork>0.5) lwork++;
-  work=(double *)calloc((size_t)lwork,sizeof(double));
+  work=(double *)R_chk_calloc((size_t)lwork,sizeof(double));
   /* actual call ... */
   F77_CALL(dormtr)(&side,&uplo,&trans,m,n,S,&nq,tau,B,m,work,&lwork,&info);
-  free(work);
+  R_chk_free(work);
 }
 
 void mgcv_tri_diag(double *S,int *n,double *tau)
@@ -427,15 +709,15 @@ void mgcv_tri_diag(double *S,int *n,double *tau)
 { int lwork=-1,info;
   double *work,work1,*e,*d;
   char uplo='U';
-  d = (double *)calloc((size_t)*n,sizeof(double));
-  e = (double *)calloc((size_t)*n-1,sizeof(double));
+  d = (double *)R_chk_calloc((size_t)*n,sizeof(double));
+  e = (double *)R_chk_calloc((size_t)*n-1,sizeof(double));
   /* work space query... */
   F77_CALL(dsytrd)(&uplo,n,S,n,d,e,tau,&work1,&lwork,&info);
   lwork=(int)floor(work1);if (work1-lwork>0.5) lwork++;
-  work=(double *)calloc((size_t)lwork,sizeof(double));
+  work=(double *)R_chk_calloc((size_t)lwork,sizeof(double));
   /* Actual call... */
   F77_CALL(dsytrd)(&uplo,n,S,n,d,e,tau,work,&lwork,&info);
-  free(work);free(d);free(e);
+  R_chk_free(work);R_chk_free(d);R_chk_free(e);
 }
 
 
@@ -505,6 +787,316 @@ void mgcv_forwardsolve(double *R,int *r,int *c,double *B,double *C, int *bc)
 }
 
 
+void row_block_reorder(double *x,int *r,int *c,int *nb,int *reverse) {
+/* x contains an r by c matrix, which is to be split into k = ceil(r/nb)
+   row-wise blocks each containing nb rows, except for the last.
+  
+   In forward mode, this splits the matrix blocks so they are stored one 
+   after another in x.
+
+   This routine re-orders these k submatrices, so that they occur one after 
+   another in x. This is done largely in-situ, with the aid of 2 indexing
+   arrays, and some extra storage required to deal with fact that last block
+   may have fewer rows than rest.
+
+   The elements of one column belonging to one block are referred to as a 
+   segment.
+*/
+  int *a,*s,k,nbf=0,i,j,q,si,ti,ns,ns_main,ns_extra;
+  double *x0,*x1,tmp,*extra=NULL;
+  k = *r / *nb; /* number of blocks */
+  if (*r > *nb * k) {
+    nbf = *r - *nb * k;k++; /* nbf number of rows in final block */
+  }
+  /* first task is to pad the end block segments, so that all segments
+     have equal length, otherwise efficient segment swapping is not
+     possible. This requires spilling over into extra storage. */
+  ns = k * *c; /* total number of segments */
+  if (nbf) { /* only do this if final block shorter than rest */
+    ns_main = (*r * *c) / *nb; /* full segments fitting in x */
+    ns_extra = ns - ns_main; /* segments requiring extra storage */
+    extra = (double *) R_chk_calloc((size_t) *nb * ns_extra,sizeof(double));
+    x0 = extra + *nb * ns_extra - 1; /* end of extra */ 
+    x1 = x + *r * *c -1 ; /* end of x */
+
+    if (*reverse) { /* blocks back into single matrix */
+      /* expand end segments out into extra storge */ 
+      for (i=ns-1;i >= ns_main;i--) {
+        x0 -= *nb - nbf; /* skip padding in target */
+        for (j=0;j<nbf;j++,x0--,x1--) *x0 = *x1; 
+      }
+      x0 = x + ns_main * *nb -1; /* target moved to main block */
+      for (;i >= ns - *c;i--) {
+        x0 -= *nb - nbf; /* skip padding in target */
+        for (j=0;j<nbf;j++,x0--,x1--) *x0 = *x1; 
+      }
+    } else { /* forward mode - matrix to separated blocks */
+      /* first copy the end segments into extra */
+      for (i=ns-1;i>=ns_main;i--) { /* work down through segments */
+        if ((i+1)%k) { /* not a short segment */     
+          for (j = 0;j < *nb;j++,x0--,x1--) *x0 = *x1;
+        } else {
+          x0 -= (*nb - nbf); /* skip padding in target */
+          for (j = 0;j < nbf;j++,x0--,x1--) *x0 = *x1; /* fill rest from source */
+        }
+      }
+      /* now copy segments into x with padding ... */
+      x0 = x + ns_main * *nb - 1; /* end of main block segment storage */
+      for (;i>=0;i--) { /* continue down through segments */
+        if ((i+1)%k) { /* not a short segment */     
+          for (j = 0;j < *nb;j++,x0--,x1--) *x0 = *x1;
+        } else {
+          x0 -= (*nb - nbf);
+          for (j = 0;j < nbf;j++,x0--,x1--) *x0 = *x1;
+        }
+      }
+    } /* end of forward mode padding */
+  } else { /* segments already equal length */
+    ns_main = ns;ns_extra=0; /* all segments fit into x */
+  }
+  /* now re-arrange row-block wise... */
+
+  /* a[i] is original segment now in segment i... */ 
+  a = (int *) R_chk_calloc((size_t) k * *c,sizeof(int));
+  /* s[i] is segment now containing original segment i */
+  s = (int *) R_chk_calloc((size_t) k * *c,sizeof(int));
+  for (i=0;i<k * *c;i++) s[i] = a[i] = i;
+  ti=0; /* target segment */
+  for (i=0;i<k;i++) for (j=0;j<*c;j++,ti++) {
+      if (*reverse) si = s[(ti%k) * *c + ti/k]; /* source segment in reverse mode */
+      else si = s[i + j * k]; /* source segment forward mode */
+      if (ti < ns_main) x0 = x + ti * *nb; /* target segment in main storage */
+      else x0 = extra + (ti - ns_main) * *nb; /* target segment in extra storage */ 
+      if (si < ns_main) x1 = x + si * *nb;
+      else x1 = extra + (si - ns_main) * *nb;
+      /* swap segment contents... */
+      for (q=0;q < *nb;q++,x0++,x1++) { tmp = *x0;*x0 = *x1;*x1 = tmp;}
+      /*  update indices... */
+      q = a[si]; a[si] = a[ti]; a[ti] = q;
+      s[a[si]] = si;s[a[ti]]=ti;
+  }  
+
+  /* now it only remains to delete padding from end block */
+  if (nbf) {
+    if (*reverse) { 
+      si = k; /* start segment */
+      x0 = x + *r; /* start target */
+      x1 = x + k * *nb; /* start source */
+      for (;si<ns;si++) {
+        if (si == ns_main) x1 = extra; /* source shifted to extra */
+        if ((si+1)%k) { /* its a full segment */
+          for (i=0; i < *nb;i++,x0++,x1++) *x0 = *x1;
+        } else { /* short segment */
+          for (i=0;i<nbf;i++,x0++,x1++) *x0 = *x1; /* only copy info, not padding */
+          x1 += *nb - nbf; /* move sourse past padding */
+        }
+      }  
+    } else { /* forward mode */
+      x1 = x0 = x + (k-1) * *c * *nb; /* address of first target (and first source) */
+      si = *c * (k-1); /* source segment index */
+      for (i=0;i < *c;i++,si++,x1+=(*nb-nbf)) {
+        if (si == ns_main) x1 = extra;  /* reset source blocks to extra */
+        for (j=0;j<nbf;j++,x0++,x1++) *x0 = *x1;   
+      }
+    }
+  } 
+
+  R_chk_free(a); R_chk_free(s);if (nbf) R_chk_free(extra);
+} /* row_block_reorder */
+
+
+int get_qpr_k(int *r,int *c,int *nt) {
+/* For a machine with nt available cores, computes k, the best number of threads to 
+   use for a parallel QR.
+*/
+  int k;double kd,fkd,x,ckd;
+  #ifdef SUPPORT_OPENMP
+  kd = sqrt(*r/(double)*c); /* theoretical optimum */
+  if (kd <= 1.0) k = 1; else
+    if (kd > *nt) k = *nt; else {
+      fkd = floor(kd);ckd = ceil(kd);
+      if (fkd>1) x =  *r / fkd + fkd * *c; else x = *r;
+      if (*r / ckd + ckd * *c < x) 
+        k = (int)ckd; else k = (int)fkd;
+    }
+  return(k);
+  #else
+  return(1); /* can only use 1 thread if no openMP support */ 
+  #endif
+}
+
+void getRpqr(double *R,double *x,int *r, int *c,int *rr,int *nt) {
+/* x contains qr decomposition of r by c matrix as computed by mgcv_pqr 
+   This routine simply extracts the c by c R factor into R. 
+   R has rr rows, where rr == c if R is square. 
+*/
+  int i,j,k,n;
+  double *Rs;
+  k = get_qpr_k(r,c,nt); /* number of blocks used */
+  if (k==1) { /* actually just a regular serial QR */ 
+    Rs = x;n = *r;
+  } else {
+    n = k * *c; /* rows of R */ 
+    Rs = x + *r * *c; /* source R */
+  }
+  for (i=0;i<*c;i++) for (j=0;j<*c;j++) if (i>j) R[i + *rr * j] = 0; else
+	R[i + *rr * j] = Rs[i + n * j];
+}
+
+void mgcv_pqrqy(double *b,double *a,double *tau,int *r,int *c,int *cb,int *tp,int *nt) {
+/* Applies factor Q of a QR factor computed in parallel to b. 
+   If b is physically r by cb, but if tp = 0 it contains a c by cb matrix on entry, while if
+   tp=1 it contains a c by cb matrix on exit. Unused elments of b on entry assumed 0. 
+   a and tau are the result of mgcv_pqr   
+*/
+  int i,j,k,l,left=1,n,nb,nbf,nq,TRUE=1,FALSE=0;
+  double *x0,*x1,*Qb;  
+  k = get_qpr_k(r,c,nt); /* number of blocks in use */
+  if (k==1) { /* single block case */ 
+    if (*tp == 0 ) {/* re-arrange so b is a full matrix */
+      x0 = b + *r * *cb -1; /* end of full b (target) */
+      x1 = b + *c * *cb -1; /* end of used block (source) */
+      for (j= *cb;j>0;j--) { /* work down columns */
+        /*for (i = *r;i > *c;i--,x0--) *x0 = 0.0;*/ /* clear unused */
+        x0 -= *r - *c; /* skip unused */
+        for (i = *c;i>0;i--,x0--,x1--) { 
+          *x0 = *x1; /* copy */
+          if (x0!=x1) *x1 = 0.0; /* clear source */
+        }
+      }
+    } /* if (*tp) */
+    mgcv_qrqy(b,a,tau,r,cb,c,&left,tp);
+    if (*tp) { /* need to strip out the extra rows */
+      x1 = x0 = b;
+      for (i=0;i < *cb;i++,x1 += *r - *c) 
+	for (j=0;j < *c;j++,x0++,x1++) *x0 = *x1; 
+    }
+    return;
+  }
+  
+  /* multi-block case starts here */
+
+  nb = (int)ceil(*r/(double)k); /* block size - in rows */
+  nbf = *r - (k-1)*nb; /* end block size */ 
+  Qb = (double *)R_chk_calloc((size_t) k * *c * *cb,sizeof(double));
+  nq = *c * k;
+  if (*tp) { /* Q'b */
+    /* first the component Q matrices are applied to the blocks of b */
+    if (*cb > 1) { /* matrix case - repacking needed */
+      row_block_reorder(b,r,cb,&nb,&FALSE);
+    }
+    #ifdef SUPPORT_OPENMP
+    #pragma omp parallel private(i,j,l,n,x1) num_threads(k)
+    #endif
+    { /* open parallel section */
+      #ifdef SUPPORT_OPENMP
+      #pragma omp for
+      #endif
+      for (i=0;i<k;i++) {
+        if (i == k-1) n = nbf; else n = nb; /* number of rows in this block */
+        x1 = b+ i * nb * *cb; /* start of current block of b */
+        mgcv_qrqy(x1,a + i * nb * *c,tau + i * *c,&n,cb,c,&left,tp);
+        /* extract the upper c by cb block into right part of Qb */
+        for (j = 0; j < *c;j++) for (l=0;l< *cb;l++) Qb[j + i * *c + nq * l] = x1[j + n * l];    
+      }
+    } /* end of parallel */      
+    /* Now multiply Qb by the combined Q' */
+    mgcv_qrqy(Qb,a + *r * *c,tau + k * *c,&nq,cb,c,&left,tp);
+    /* and finally copy the top c rows of Qb into b */
+    x0 = b; /* target */
+    x1 = Qb; /* source */
+    for (i=0;i<*cb;i++) { 
+      for (j=0;j<*c;j++,x0++,x1++) *x0 = *x1;
+      x1 += (k-1) * *c; /* skip rows of Qb */ 
+    }
+  } else {  /* Qb */
+    /* copy c by cb matrix at start of b into first c rows of Qb */
+    x0=Qb; /* target */
+    x1=b;
+    for (i=0;i<*cb;i++) {
+      for (j=0;j<*c;j++,x0++,x1++) {
+        *x0 = *x1;
+        *x1=0.0; /* erase or left in below */
+      }
+      x0 += (k-1) * *c; /* skip rows of Qb */ 
+    }
+    /* now apply the combined Q factor */
+    mgcv_qrqy(Qb,a + *r * *c,tau + k * *c,&nq,cb,c,&left,tp);
+    /* the blocks of Qb now have to be copied into separate blocks in b */
+    #ifdef SUPPORT_OPENMP
+    #pragma omp parallel private(i,j,l,n,x0,x1) num_threads(k)
+    #endif
+    { /* open parallel section */
+      #ifdef SUPPORT_OPENMP
+      #pragma omp for
+      #endif
+      for (i=0;i<k;i++) {
+        if (i == k-1) n = nbf; else n = nb; /* number of rows in this block */
+        x0 = b + nb * *cb * i; /* target block start */
+        x1 = Qb + i * *c; /* source row start */
+        for (j=0;j < *cb;j++) { /* across the cols */
+          for (l=0;l< *c;l++,x0++,x1++) *x0 = *x1;
+          x0 += n - *c; /* to start of next column */
+          x1 += nq - *c; /* ditto */ 
+        }
+        x1 = b+ i * nb * *cb; /* start of current block of b */
+        mgcv_qrqy(x1,a + i * nb * *c,tau + i * *c,&n,cb,c,&left,tp);
+      }
+    } /* end of parallel section */
+    /* now need to unscramble separate blocks back into b as regular matrix */
+    if (*cb>1) row_block_reorder(b,r,cb,&nb,&TRUE);
+  }
+  R_chk_free(Qb);
+} 
+
+void mgcv_pqr(double *x,int *r, int *c,int *pivot, double *tau, int *nt) {
+/* parallel qr decomposition using up to nt threads. 
+   * x is an r*c+nt*c^2 array. On entry first r*c entries is matrix to decompose.
+   * pivot is a c vector and tau is a (nt+1)*c vector.
+   On exit x, pivot and tau contain decompostion information in packed form
+   for use by getRpqr and mgcv_pqrqy
+   Routine first computes k, the optimal number of threads to use from 1 to nt.
+*/
+
+  int i,j,k,l,*piv,nb,nbf,n,TRUE=1,FALSE=0,nr; 
+  double *R,*R1,*xi; 
+ 
+  k = get_qpr_k(r,c,nt);/* number of threads to use */
+ 
+  if (k==1) mgcv_qr(x,r,c,pivot,tau); else { /* multi-threaded version */
+    nb = (int)ceil(*r/(double)k); /* block size */
+    nbf = *r - (k-1)*nb; /* end block size */
+    /* need to re-arrange row blocks so that they can be split between qr calls */
+    row_block_reorder(x,r,c,&nb,&FALSE); 
+    piv = (int *)R_chk_calloc((size_t) k * *c,sizeof(int));
+    R = x + *r * *c ; /* pointer to combined unpivoted R matrix */
+    nr = *c * k; /* number of rows in R */
+    #ifdef SUPPORT_OPENMP
+    #pragma omp parallel private(i,j,l,n,xi,R1) num_threads(k)
+    #endif
+    { /* open parallel section */
+      #ifdef SUPPORT_OPENMP
+      #pragma omp for
+      #endif
+      for (i=0;i<k;i++) { /* QR the blocks */
+        if (i==k-1) n = nbf; else n = nb; 
+        xi = x + nb * i * *c; /* current block */
+        mgcv_qr(xi,&n,c,piv + i * *c,tau + i * *c);
+        /* and copy the R factors, unpivoted into a new matrix */
+        R1 = (double *)R_chk_calloc((size_t)*c * *c,sizeof(double));
+        for (l=0;l<*c;l++) for (j=l;j<*c;j++) R1[l + *c * j] = xi[l + n * j]; 
+        /* What if final nbf is less than c? - can't happen  */
+        pivoter(R1,c,c,piv + i * *c,&TRUE,&TRUE); /* unpivoting the columns of R1 */
+        for (l=0;l<*c;l++) for (j=0;j<*c;j++) R[i * *c +l + nr *j] = R1[l+ *c * j];
+        R_chk_free(R1);
+      }
+    } /* end of parallel section */
+    R_chk_free(piv);
+    n = k * *c;
+    mgcv_qr(R,&n,c,pivot,tau + k * *c); /* final pivoted QR */
+  }
+}
+
 void mgcv_qr(double *x, int *r, int *c,int *pivot,double *tau)
 /* call LA_PACK to get pivoted QR decomposition of x
    tau is an array of length min(r,c)
@@ -526,10 +1118,10 @@ void mgcv_qr(double *x, int *r, int *c,int *pivot,double *tau)
   /* Args: M, N, A, LDA, JPVT, TAU, WORK, LWORK, INFO */
   F77_CALL(dgeqp3)(r,c,x,r,pivot,tau,&work1,&lwork,&info);
   lwork=(int)floor(work1);if (work1-lwork>0.5) lwork++;
-  work=(double *)calloc((size_t)lwork,sizeof(double));
+  work=(double *)R_chk_calloc((size_t)lwork,sizeof(double));
    /* actual call */
   F77_CALL(dgeqp3)(r,c,x,r,pivot,tau,work,&lwork,&info); 
-  free(work);
+  R_chk_free(work);
   /*if (*r<*c) lwork= *r; else lwork= *c;*/ 
   for (ip=pivot;ip < pivot + *c;ip++) (*ip)--;
   /* ... for 'tis C in which we work and not the 'cursed Fortran... */
@@ -552,11 +1144,11 @@ void mgcv_qr2(double *x, int *r, int *c,int *pivot,double *tau)
 */
 { int info,*ip,i;
   double *work;
-  work=(double *)calloc((size_t)*r,sizeof(double));
+  work=(double *)R_chk_calloc((size_t)*r,sizeof(double));
    /* actual call */
   /* Args: M, N, A, LDA, TAU, WORK, INFO */
   F77_CALL(dgeqr2)(r,c,x,r,tau,work,&info); 
-  free(work);
+  R_chk_free(work);
   /*if (*r<*c) lwork= *r; else lwork= *c;*/ 
   for (i=0,ip=pivot;ip < pivot + *c;ip++,i++) *ip = i;
   /* ... pivot index equivalent to no pivoting */
@@ -592,10 +1184,10 @@ void mgcv_qrqy(double *b,double *a,double *tau,int *r,int *c,int *k,int *left,in
   
   F77_CALL(dormqr)(&side,&trans,r,c,k,a,&lda,tau,b,r,&work1,&lwork,&info);
   lwork=(int)floor(work1);if (work1-lwork>0.5) lwork++;
-  work=(double *)calloc((size_t)lwork,sizeof(double));
+  work=(double *)R_chk_calloc((size_t)lwork,sizeof(double));
   /* actual call */
   F77_CALL(dormqr)(&side,&trans,r,c,k,a,&lda,tau,b,r,work,&lwork,&info); 
-  free(work);
+  R_chk_free(work);
    
 }
 
@@ -642,8 +1234,8 @@ qr.R(qr(Xa,tol=0))
 */
 
 { double *x,*work,c,s,r,x0,x1,m,*xip,*xjp,*riip,*rijp,*Qp,*wp;
-  x=(double *)calloc((size_t)*q,sizeof(double)); 
-  work=(double *)calloc((size_t)*n,sizeof(double)); /* working extra column of Q */
+  x=(double *)R_chk_calloc((size_t)*q,sizeof(double)); 
+  work=(double *)R_chk_calloc((size_t)*n,sizeof(double)); /* working extra column of Q */
   x[*k] = *lam;
   /* conceptually i runs from k to q in the following loop */
   for (Qp=Q+ *k * *n,riip=R+ *k * *q + *k,xip=x+ *k ;xip< x+ *q;xip++,riip+= *q+1)
@@ -671,7 +1263,7 @@ qr.R(qr(Xa,tol=0))
       *wp = s*x1+c* *wp;
     }
   }
-  free(x);free(work);
+  R_chk_free(x);R_chk_free(work);
 }
 
 void mgcv_symeig(double *A,double *ev,int *n,int *use_dsyevd,int *get_vectors,
@@ -708,7 +1300,7 @@ void mgcv_symeig(double *A,double *ev,int *n,int *use_dsyevd,int *get_vectors,
   double work1,*work,dum1=0,abstol=0.0,*Z,*dum2,x,*p,*p1,*p2,*Acopy;
   int lwork = -1,liwork = -1,iwork1,info,*iwork,dumi=0,n_eval=0,*isupZ,i,j,k,debug=0;
   if (debug && *get_vectors) { /* need a copy to dump in case of trouble */
-    Acopy = (double *)calloc((size_t)*n * *n,sizeof(double));
+    Acopy = (double *)R_chk_calloc((size_t)*n * *n,sizeof(double));
     for (p2=Acopy,p=A,p1=A+ *n * *n;p<p1;p++,p2++) *p2 = *p;
   }
 
@@ -716,10 +1308,10 @@ void mgcv_symeig(double *A,double *ev,int *n,int *use_dsyevd,int *get_vectors,
   if (*use_dsyevd)
   { F77_CALL(dsyevd)(&jobz,&uplo,n,A,n,ev,&work1,&lwork,&iwork1,&liwork,&info);
     lwork=(int)floor(work1);if (work1-lwork>0.5) lwork++;
-    work=(double *)calloc((size_t)lwork,sizeof(double));
-    liwork = iwork1;iwork= (int *)calloc((size_t)liwork,sizeof(int));
+    work=(double *)R_chk_calloc((size_t)lwork,sizeof(double));
+    liwork = iwork1;iwork= (int *)R_chk_calloc((size_t)liwork,sizeof(int));
     F77_CALL(dsyevd)(&jobz,&uplo,n,A,n,ev,work,&lwork,iwork,&liwork,&info);
-    free(work);free(iwork);
+    R_chk_free(work);R_chk_free(iwork);
     if (*descending) for (i=0;i<*n/2;i++) {
         /* work in from left and right swapping cols */
 	p = A + i * *n; /* start of left col */ 
@@ -729,20 +1321,20 @@ void mgcv_symeig(double *A,double *ev,int *n,int *use_dsyevd,int *get_vectors,
         }
     }
   } else
-  { Z=(double *)calloc((size_t)(*n * *n),sizeof(double)); /* eigen-vector storage */
-    isupZ=(int *)calloc((size_t)(2 * *n),sizeof(int)); /* eigen-vector support */
+  { Z=(double *)R_chk_calloc((size_t)(*n * *n),sizeof(double)); /* eigen-vector storage */
+    isupZ=(int *)R_chk_calloc((size_t)(2 * *n),sizeof(int)); /* eigen-vector support */
     F77_CALL(dsyevr)(&jobz,&range,&uplo,
 		   n,A,n,&dum1,&dum1,&dumi,&dumi,
 		   &abstol,&n_eval,ev, 
     		   Z,n,isupZ, &work1,&lwork,&iwork1,&liwork,&info);
     lwork=(int)floor(work1);if (work1-lwork>0.5) lwork++;
-    work=(double *)calloc((size_t)lwork,sizeof(double));
-    liwork = iwork1;iwork= (int *)calloc((size_t)liwork,sizeof(int));
+    work=(double *)R_chk_calloc((size_t)lwork,sizeof(double));
+    liwork = iwork1;iwork= (int *)R_chk_calloc((size_t)liwork,sizeof(int));
     F77_CALL(dsyevr)(&jobz,&range,&uplo,
 		   n,A,n,&dum1,&dum1,&dumi,&dumi,
 		   &abstol,&n_eval,ev, 
     		     Z,n,isupZ, work,&lwork,iwork,&liwork,&info);
-    free(work);free(iwork);
+    R_chk_free(work);R_chk_free(iwork);
     
     /* if (*descending) for (i=0;i<*n/2;i++) { 
       x = ev[i]; ev[i] = ev[*n-i-1];ev[*n-i-1] = x;
@@ -760,13 +1352,13 @@ void mgcv_symeig(double *A,double *ev,int *n,int *use_dsyevd,int *get_vectors,
       }
     }
     A = p1;
-    free(Z);free(isupZ);
+    R_chk_free(Z);R_chk_free(isupZ);
   }
   if (*descending) for (i=0;i<*n/2;i++) { /* reverse the eigenvalues */
        x = ev[i]; ev[i] = ev[*n-i-1];ev[*n-i-1] = x;
   }
   if (debug && *get_vectors) { /* are the eigenvectors really orthogonal?? */
-    p = (double *)calloc((size_t)*n * *n,sizeof(double));
+    p = (double *)R_chk_calloc((size_t)*n * *n,sizeof(double));
     getXtX(p,A,n,n); /* cross prod of eigenvec matrix - should be I */
     x=0.0;k=0;
     for (i=0;i<*n;i++) for (j=0;j<i;j++)  if (fabs(p[i + *n * j])>1e-14) { 
@@ -781,7 +1373,7 @@ void mgcv_symeig(double *A,double *ev,int *n,int *use_dsyevd,int *get_vectors,
     }
     if (k) Rprintf("Eigenvectors not normalized %d %g\n",k,x/k);
     if (k+j>0) dump_mat(Acopy,n,n,"/home/sw283/tmp/badmat.dat");
-    free(p);free(Acopy);
+    R_chk_free(p);R_chk_free(Acopy);
   }
 
 }
@@ -814,9 +1406,9 @@ void mgcv_trisymeig(double *d,double *g,double *v,int *n,int getvec,int descendi
 		   &iwork1, &liwork, &info);
 
    lwork=(int)floor(work1);if (work1-lwork>0.5) lwork++;
-   work=(double *)calloc((size_t)lwork,sizeof(double));
+   work=(double *)R_chk_calloc((size_t)lwork,sizeof(double));
    liwork = iwork1;
-   iwork= (int *)calloc((size_t)liwork,sizeof(int));
+   iwork= (int *)R_chk_calloc((size_t)liwork,sizeof(int));
 
    /* and the actual call... */
    F77_CALL(dstedc)(&compz,n,
@@ -836,7 +1428,7 @@ void mgcv_trisymeig(double *d,double *g,double *v,int *n,int getvec,int descendi
      }
    }
 
-   free(work);free(iwork);
+   R_chk_free(work);R_chk_free(iwork);
    *n=info; /* zero is success */
 }
 
@@ -872,10 +1464,10 @@ void Rlanczos(double *A,double *U,double *D,int *n, int *m, int *lm,double *tol)
   kk = (int) floor(*n/10); if (kk<1) kk=1;  
   if (kk<f_check) f_check = kk;
 
-  q=(double **)calloc((size_t)(*n+1),sizeof(double *));
+  q=(double **)R_chk_calloc((size_t)(*n+1),sizeof(double *));
 
   /* "randomly" initialize first q vector */
-  q[0]=(double *)calloc((size_t)*n,sizeof(double));
+  q[0]=(double *)R_chk_calloc((size_t)*n,sizeof(double));
   b=q[0];bt=0.0;
   for (i=0;i < *n;i++)   /* somewhat randomized start vector!  */
   { jran=(jran*ia+ic) % im;   /* quick and dirty generator to avoid too regular a starting vector */
@@ -886,12 +1478,12 @@ void Rlanczos(double *A,double *U,double *D,int *n, int *m, int *lm,double *tol)
   for (i=0;i < *n;i++) b[i]/=bt;
 
   /* initialise vectors a and b - a is leading diagonal of T, b is sub/super diagonal */
-  a=(double *)calloc((size_t) *n,sizeof(double));
-  b=(double *)calloc((size_t) *n,sizeof(double));
-  g=(double *)calloc((size_t) *n,sizeof(double));
-  d=(double *)calloc((size_t) *n,sizeof(double)); 
-  z=(double *)calloc((size_t) *n,sizeof(double));
-  err=(double *)calloc((size_t) *n,sizeof(double));
+  a=(double *)R_chk_calloc((size_t) *n,sizeof(double));
+  b=(double *)R_chk_calloc((size_t) *n,sizeof(double));
+  g=(double *)R_chk_calloc((size_t) *n,sizeof(double));
+  d=(double *)R_chk_calloc((size_t) *n,sizeof(double)); 
+  z=(double *)R_chk_calloc((size_t) *n,sizeof(double));
+  err=(double *)R_chk_calloc((size_t) *n,sizeof(double));
   for (i=0;i< *n;i++) err[i]=1e300;
   /* The main loop. Will break out on convergence. */
   for (j=0;j< *n;j++) 
@@ -946,7 +1538,7 @@ void Rlanczos(double *A,double *U,double *D,int *n, int *m, int *lm,double *tol)
     /*if (b[j]==0.0&&j< *n-1) ErrorMessage(_("Lanczos failed"),1);*/ /* Actually this isn't really a failure => rank(A)<l+lm */
     /* get q[j+1]      */
     if (j < *n-1)
-    { q[j+1]=(double *)calloc((size_t) *n,sizeof(double));
+    { q[j+1]=(double *)R_chk_calloc((size_t) *n,sizeof(double));
       for (xx=b[j],qp=q[j+1],p0=qp + *n,zp=z;qp<p0;qp++,zp++) *qp = *zp/xx; /* forming q[j+1]=z/b[j] */
     }
 
@@ -956,9 +1548,9 @@ void Rlanczos(double *A,double *U,double *D,int *n, int *m, int *lm,double *tol)
     { for (i=0;i<j+1;i++) d[i]=a[i]; /* copy leading diagonal of T_j */
       for (i=0;i<j;i++) g[i]=b[i]; /* copy sub/super diagonal of T_j */   
       /* set up storage for eigen vectors */
-      if (vlength) free(v); /* free up first */
+      if (vlength) R_chk_free(v); /* free up first */
       vlength=j+1; 
-      v = (double *)calloc((size_t)vlength*vlength,sizeof(double));
+      v = (double *)R_chk_calloc((size_t)vlength*vlength,sizeof(double));
  
       /* obtain eigen values/vectors of T_j in O(j^2) flops */
     
@@ -1028,14 +1620,14 @@ void Rlanczos(double *A,double *U,double *D,int *n, int *m, int *lm,double *tol)
   }
  
   /* clean up..... */
-  free(a);
-  free(b);
-  free(g);
-  free(d);
-  free(z);
-  free(err);
-  if (vlength) free(v);
-  for (i=0;i< *n+1;i++) if (q[i]) free(q[i]);free(q);  
+  R_chk_free(a);
+  R_chk_free(b);
+  R_chk_free(g);
+  R_chk_free(d);
+  R_chk_free(z);
+  R_chk_free(err);
+  if (vlength) R_chk_free(v);
+  for (i=0;i< *n+1;i++) if (q[i]) R_chk_free(q[i]);R_chk_free(q);  
   *n = j; /* number of iterations taken */
 } /* end of Rlanczos */
 
