@@ -1,8 +1,8 @@
 /* main method routines */
-
+#include <Rinternals.h>
 void magic(double *y,double *X,double *sp0,double *def_sp,double *S,double *H,double *L,
 	   double *lsp0,double *gamma,double *scale, int *control,int *cS,double *rank_tol,
-	   double *tol,double *b,double *rV,double *norm_const,int *n_score);
+	   double *tol,double *b,double *rV,double *norm_const,int *n_score,int *nt);
 
 void gdi1(double *X,double *E,double *Es,double *rS,double *U1,
 	  double *sp,double *z,double *w,double *wf,double *alpha,double *mu,double *eta, double *y,
@@ -11,12 +11,21 @@ void gdi1(double *X,double *E,double *Es,double *rS,double *U1,
          double *P0, double *P1,double *P2,double *trA,
          double *trA1,double *trA2,double *rV,double *rank_tol,double *conv_tol, int *rank_est,
 	 int *n,int *q, int *M,int *Mp,int *Enrow,int *rSncol,int *deriv,
-	  int *REML,int *fisher,int *fixed_penalty);     
+	  int *REML,int *fisher,int *fixed_penalty,int *nthreads);     
+
+void gdi2(double *X,double *E,double *Es,double *rS,double *U1,
+	  double *sp,double *theta,double *z,double *w,
+          double *Dth,double *Det,double *Det2,double *Dth2,double *Det_th,
+          double *Det2_th,double *Det3,double *Det_th2,
+          double *Det4, double *Det3_th, double *Det2_th2,
+          double *beta,double *D1,double *D2,double *P,double *P1,double *P2,
+          double *ldet, double *ldet1,double *ldet2,
+          double *rank_tol,int *rank_est,
+	  int *n,int *q, int *M,int *n_theta, int *Mp,int *Enrow,int *rSncol,int *deriv,
+	  int *fixed_penalty,int *nt);
 
-void pls_fit(double *y,double *X,double *w,double *E,int *n,int *q,int *cE,double *eta,
-             double *penalty,double *rank_tol);
 void pls_fit1(double *y,double *X,double *w,double *E,double *Es,int *n,int *q,int *rE,double *eta,
-	      double *penalty,double *rank_tol);
+	      double *penalty,double *rank_tol,int *nt);
 
 void get_detS2(double *sp,double *sqrtS, int *rSncol, int *q,int *M, int * deriv, 
                double *det, double *det1, double *det2, double *d_tol,
@@ -32,7 +41,6 @@ void psum(double *y, double *x,int *index,int *n);
 void rwMatrix(int *stop,int *row,double *w,double *X,int *n,int *p);
 void in_out(double *bx, double *by, double *break_code, double *x,double *y,int *in, int *nb, int *n);
 void Rlanczos(double *A,double *U,double *D,int *n, int *m, int *lm,double *tol);
-void RQT(double *A,int *r,int *c);
 void RuniqueCombs(double *X,int *ind,int *r, int *c);
 void  RPCLS(double *Xd,double *pd,double *yd, double *wd,double *Aind,double *bd,double *Afd,double *Hd,double *Sd,int *off,int *dim,double *theta, int *m,int *nar);
 void RMonoCon(double *Ad,double *bd,double *xd,int *control,double *lower,double *upper,int *n);
@@ -40,7 +48,7 @@ void MinimumSeparation(double *gx,double *gy,int *gn,double *dx,double *dy, int
 void rksos(double *x,int *n,double *eps);
 void pivoter(double *x,int *r,int *c,int *pivot, int *col, int *reverse);
 
-/* Routines for direct access to linpack and lapack */
+/* Routines for linear algebra with direct access to linpack and lapack */
 
 void mgcv_chol(double *a,int *pivot,int *n,int *rank);
 void mgcv_svd(double *x,double *u, double *d,int *r,int *c);
@@ -51,6 +59,8 @@ void mgcv_qr(double *x, int *r, int *c,int *pivot,double *tau);
 void mgcv_qr2(double *x, int *r, int *c,int *pivot,double *tau);
 void update_qr(double *Q,double *R,int *n, int *q,double *lam, int *k);
 extern void mgcv_mmult(double *A,double *B,double *C,int *bt,int *ct,int *r,int *c,int *n);
+void mgcv_pmmult(double *A,double *B,double *C,int *bt,int *ct,int *r,int *c,int *n,int *nt);
+SEXP mgcv_pmmult2(SEXP b, SEXP c,SEXP bt,SEXP ct, SEXP nthreads);
 void mgcv_mmult0(double *A,double *B,double *C,int *bt,int *ct,int *r,int *c,int *n);
 void mgcv_svd_full(double *x,double *vt,double *d,int *r,int *c);
 void mgcv_symeig(double *A,double *ev,int *n,int *use_dsyevd, int *get_vectors,int *descending);
@@ -64,6 +74,11 @@ void getXtX(double *XtX,double *X,int *r,int *c);
 void getXtMX(double *XtMX,double *X,double *M,int *r,int *c,double *work);
 void getXXt(double *XXt,double *X,int *r,int *c);
 void read_mat(double *M,int *r,int*c, char *path);
+void row_block_reorder(double *x,int *r,int *c,int *nb,int *reverse);
+void mgcv_pqr(double *x,int *r, int *c,int *pivot, double *tau, int *nt);
+void getRpqr(double *R,double *x,int *r, int *c,int *rr,int *nt);
+void mgcv_pqrqy(double *b,double *a,double *tau,int *r,int *c,int *cb,int *tp,int *nt);
+SEXP mgcv_Rpiqr(SEXP X, SEXP BETA,SEXP PIV,SEXP NT);
 
 
 /* basis constructor/prediction routines*/
@@ -81,11 +96,11 @@ void gridder(double *z,double *x,double *y,int *n,double *g, int *G,int *nx, int
 void pde_coeffs(int *G,double *x,int *ii,int *jj,int *n,int *nx,int *ny,double *dx,double *dy);
 
 /* sparse smooth related routines */
-void sparse_penalty(double *X,int *n,int *d,double *D,int *ni,int *k,int *m,int *a_weight,double *kappa);
 void k_nn(double *X,double *dist,double *a,int *ni,int *n,int *d,int *k,int *get_a);
-void kba_nn(double *X,double *dist,double *a,int *ni,int *n,int *d,int *k,
-            int *get_a,double *cut_off);
-void Rkdtree(double *X,int *n, int *d,double *lo,double *hi,int *ind, int *rind);
+void Rkdtree(double *X,int *n, int *d,int *idat,double *ddat);
+void Rkdnearest(double *X,int *idat,double *ddat,int *n,double *x, int *m, int *ni, double *dist,int *k);
+void Rkradius(double *r,int *idat,double *ddat,double *X,double *x,int *m,int *off,int *ni,int *op);
+
 void tri2nei(int *t,int *nt,int *n,int *d,int *off);
 void nei_penalty(double *X,int *n,int *d,double *D,int *ni,int *ii,int *off,
 		 int *m,int *a_weight,double *kappa);
